@@ -31,19 +31,12 @@
 #include <errno.h>
 
 #include <glib/gi18n-lib.h>
-#include <cairo.h>
-#include <cairo-gobject.h>
-#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include <gtk/gtkx.h>
-#include <X11/Xatom.h>
-#include <cairo-xlib.h>
 
 #include "panel-applet.h"
 #include "panel-applet-private.h"
 #include "panel-applet-bindings.h"
 #include "panel-applet-factory.h"
-#include "panel-applet-marshal.h"
 #include "panel-applet-enums.h"
 
 /**
@@ -94,14 +87,11 @@ struct _PanelAppletPrivate {
         int               *size_hints;
         int                size_hints_len;
 
-	gboolean           moving_focus_out;
-
 	gboolean           locked_down;
 };
 
 enum {
         CHANGE_ORIENT,
-	MOVE_FOCUS_OUT_OF_APPLET,
         LAST_SIGNAL
 };
 
@@ -507,90 +497,6 @@ panel_applet_set_locked_down (PanelApplet *applet,
 	g_object_notify (G_OBJECT (applet), "locked-down");
 }
 
-static Atom _net_wm_window_type = None;
-static Atom _net_wm_window_type_dock = None;
-static Atom _net_active_window = None;
-
-static void
-panel_applet_init_atoms (Display *xdisplay)
-{
-	if (_net_wm_window_type == None)
-		_net_wm_window_type = XInternAtom (xdisplay,
-						   "_NET_WM_WINDOW_TYPE",
-						   False);
-
-	if (_net_wm_window_type_dock == None)
-		_net_wm_window_type_dock = XInternAtom (xdisplay,
-							"_NET_WM_WINDOW_TYPE_DOCK",
-							False);
-
-	if (_net_active_window == None)
-		_net_active_window = XInternAtom (xdisplay,
-						  "_NET_ACTIVE_WINDOW",
-						  False);
-}
-
-static Window
-panel_applet_find_toplevel_dock_window (PanelApplet *applet,
-					Display	    *xdisplay)
-{
-	GtkWidget  *toplevel;
-	Window	    xwin;
-	Window	    root, parent, *child;
-	int	    num_children;
-
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (applet));
-	if (!gtk_widget_get_realized (toplevel))
-		return None;
-
-	xwin = GDK_WINDOW_XID (gtk_widget_get_window (toplevel));
-
-	child = NULL;
-	parent = root = None;
-	do {
-		Atom	type_return;
-		Atom	window_type;
-		int	format_return;
-		gulong	number_return, bytes_after_return;
-		guchar *data_return;
-
-		XGetWindowProperty (xdisplay,
-				    xwin,
-				    _net_wm_window_type,
-				    0, 1, False,
-				    XA_ATOM,
-				    &type_return, &format_return,
-				    &number_return,
-				    &bytes_after_return,
-				    &data_return);
-
-		if (type_return == XA_ATOM) {
-			window_type = *(Atom *) data_return;
-
-			XFree (data_return);
-			data_return = NULL;
-
-			if (window_type == _net_wm_window_type_dock)
-				return xwin;
-		}
-
-		if (!XQueryTree (xdisplay,
-			   xwin,
-			   &root, &parent, &child,
-			   (guint *) &num_children)) {
-			   return None;
-		}
-
-		if (child && num_children > 0)
-			XFree (child);
-
-		xwin = parent;
-
-	} while (xwin != None && xwin != root);
-
-	return None;
-}
-
 /**
  * panel_applet_request_focus:
  * @applet: a #PanelApplet.
@@ -600,55 +506,22 @@ panel_applet_find_toplevel_dock_window (PanelApplet *applet,
  * Requests focus for @applet. There is no guarantee that @applet will
  * successfully get focus after that call.
  **/
-/* This function
- *   1) Gets the window id of the panel that contains the applet
- *	using XQueryTree and XGetWindowProperty to find an ancestor
- *	window with the _NET_WM_WINDOW_TYPE_DOCK window type.
- *   2) Sends a _NET_ACTIVE_WINDOW message to get that panel focused
- */
 void
 panel_applet_request_focus (PanelApplet	 *applet,
 			    guint32	  timestamp)
 {
-	GdkScreen  *screen;
-	GdkWindow  *root;
-	GdkDisplay *display;
-	Display	   *xdisplay;
-	Window	    dock_xwindow;
-	Window	    xroot;
-	XEvent	    xev;
+  GtkWidget *toplevel;
+  GdkWindow *window;
 
-	g_return_if_fail (PANEL_IS_APPLET (applet));
+  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (applet));
+  if (!toplevel)
+    return;
 
-	screen	= gtk_widget_get_screen (GTK_WIDGET (applet));
-	root	= gdk_screen_get_root_window (screen);
-	display = gdk_screen_get_display (screen);
+  window = gtk_widget_get_window (toplevel);
+  if (!window)
+    return;
 
-	xdisplay = GDK_DISPLAY_XDISPLAY (display);
-	xroot	 = GDK_WINDOW_XID (root);
-
-	panel_applet_init_atoms (xdisplay);
-
-	dock_xwindow = panel_applet_find_toplevel_dock_window (applet, xdisplay);
-	if (dock_xwindow == None)
-		return;
-
-	xev.xclient.type	 = ClientMessage;
-	xev.xclient.serial	 = 0;
-	xev.xclient.send_event	 = True;
-	xev.xclient.window	 = dock_xwindow;
-	xev.xclient.message_type = _net_active_window;
-	xev.xclient.format	 = 32;
-	xev.xclient.data.l[0]	 = 1; /* requestor type; we're an app, I guess */
-	xev.xclient.data.l[1]	 = timestamp;
-	xev.xclient.data.l[2]	 = None; /* "currently active window", supposedly */
-	xev.xclient.data.l[3]	 = 0;
-	xev.xclient.data.l[4]	 = 0;
-
-	XSendEvent (xdisplay,
-		    xroot, False,
-		    SubstructureRedirectMask | SubstructureNotifyMask,
-		    &xev);
+  gdk_window_focus (window, timestamp);
 }
 
 static GAction *
@@ -913,89 +786,9 @@ container_has_focusable_child (GtkContainer *container)
 	return retval;
 }
 
-static void
-panel_applet_position_menu (GtkMenu   *menu,
-			    int       *x,
-			    int       *y,
-			    gboolean  *push_in,
-			    GtkWidget *widget)
-{
-	PanelApplet    *applet;
-	GtkAllocation   allocation;
-	GtkRequisition  requisition;
-	GdkDisplay     *display;
-	GdkSeat        *seat;
-	GdkDevice      *device;
-	GdkWindow      *window;
-	GdkScreen      *screen;
-	int             menu_x = 0;
-	int             menu_y = 0;
-	int             pointer_x;
-	int             pointer_y;
-
-	g_return_if_fail (PANEL_IS_APPLET (widget));
-
-	applet = PANEL_APPLET (widget);
-
-	display = gtk_widget_get_display (widget);
-	seat = gdk_display_get_default_seat (display);
-	device = gdk_seat_get_pointer (seat);
-	screen = gtk_widget_get_screen (widget);
-	window = gtk_widget_get_window (widget);
-
-	gtk_menu_set_screen (menu, screen);
-
-	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition, NULL);
-	gdk_window_get_origin (window, &menu_x, &menu_y);
-	gdk_window_get_device_position (window, device, &pointer_x, &pointer_y, NULL);
-	gtk_widget_get_allocation (widget, &allocation);
-
-	menu_x += allocation.x;
-	menu_y += allocation.y;
-
-	if (applet->priv->orient == PANEL_APPLET_ORIENT_UP ||
-	    applet->priv->orient == PANEL_APPLET_ORIENT_DOWN) {
-		if (gtk_widget_get_direction (GTK_WIDGET (menu)) != GTK_TEXT_DIR_RTL) {
-			if (pointer_x < allocation.width &&
-			    requisition.width < pointer_x)
-				menu_x += MIN (pointer_x,
-					       allocation.width - requisition.width);
-		} else {
-			menu_x += allocation.width - requisition.width;
-			if (pointer_x > 0 && pointer_x < allocation.width &&
-			    pointer_x < allocation.width - requisition.width) {
-				menu_x -= MIN (allocation.width - pointer_x,
-					       allocation.width - requisition.width);
-			}
-		}
-		menu_x = MIN (menu_x, gdk_screen_get_width (screen) - requisition.width);
-
-		if (menu_y > gdk_screen_get_height (screen) / 2)
-			menu_y -= requisition.height;
-		else
-			menu_y += allocation.height;
-	} else  {
-		if (pointer_y < allocation.height &&
-		    requisition.height < pointer_y)
-			menu_y += MIN (pointer_y, allocation.height - requisition.height);
-		menu_y = MIN (menu_y, gdk_screen_get_height (screen) - requisition.height);
-
-		if (menu_x > gdk_screen_get_width (screen) / 2)
-			menu_x -= requisition.width;
-		else
-			menu_x += allocation.width;
-
-	}
-
-	*x = menu_x;
-	*y = menu_y;
-	*push_in = FALSE;
-}
-
 static void
 panel_applet_menu_popup (PanelApplet *applet,
-			 guint        button,
-			 guint32      time)
+                         GdkEvent    *event)
 {
 	GtkWidget *menu;
 	GMenu     *gmenu;
@@ -1018,17 +811,16 @@ panel_applet_menu_popup (PanelApplet *applet,
 	if (!visible)
 		return;
 
-	gtk_menu_popup (GTK_MENU (menu),
-			NULL, NULL,
-			(GtkMenuPositionFunc) panel_applet_position_menu,
-			applet,
-			button, time);
+	gtk_menu_popup_at_widget (GTK_MENU (menu),
+	                          GTK_WIDGET (applet),
+	                          GDK_GRAVITY_SOUTH_WEST,
+	                          GDK_GRAVITY_NORTH_WEST,
+	                          event);
 }
 
 static void
 panel_applet_edit_menu_popup (PanelApplet *applet,
-			      guint        button,
-			      guint32      time)
+                              GdkEvent    *event)
 {
 	GtkWidget *menu;
 	GMenu     *gmenu;
@@ -1040,13 +832,11 @@ panel_applet_edit_menu_popup (PanelApplet *applet,
 	menu = gtk_menu_new_from_model (G_MENU_MODEL (gmenu));
 
 	gtk_menu_attach_to_widget (GTK_MENU (menu), GTK_WIDGET (applet), NULL);
-
-	gtk_menu_set_screen (GTK_MENU (menu), gtk_widget_get_screen (GTK_WIDGET (applet)));
-	gtk_menu_popup (GTK_MENU (menu),
-			NULL, NULL,
-			NULL,
-			applet,
-			button, time);
+	gtk_menu_popup_at_widget (GTK_MENU (menu),
+	                          GTK_WIDGET (applet),
+	                          GDK_GRAVITY_SOUTH_WEST,
+	                          GDK_GRAVITY_NORTH_WEST,
+	                          event);
 }
 
 static gboolean
@@ -1084,9 +874,9 @@ panel_applet_button_press (GtkWidget      *widget,
 		modifiers = event->state & gtk_accelerator_get_default_mod_mask ();
 
 		if (modifiers == panel_applet_bindings_get_mouse_button_modifier_keymask ())
-			panel_applet_edit_menu_popup (applet, event->button, event->time);
+			panel_applet_edit_menu_popup (applet, (GdkEvent *) event);
 		else
-			panel_applet_menu_popup (applet, event->button, event->time);
+			panel_applet_menu_popup (applet, (GdkEvent *) event);
 
 		return TRUE;
 	}
@@ -1119,9 +909,9 @@ panel_applet_key_press_event (GtkWidget   *widget,
 	panel_applet_bindings_key_event_is_popup (event, &is_popup, &is_edit_popup);
 
 	if (is_edit_popup)
-		panel_applet_edit_menu_popup (PANEL_APPLET (widget), 3, event->time);
+		panel_applet_edit_menu_popup (PANEL_APPLET (widget), (GdkEvent *) event);
 	else if (is_popup)
-		panel_applet_menu_popup (PANEL_APPLET (widget), 3, event->time);
+		panel_applet_menu_popup (PANEL_APPLET (widget), (GdkEvent *) event);
 
 	return (is_popup || is_edit_popup);
 }
@@ -1217,19 +1007,6 @@ panel_applet_focus (GtkWidget        *widget,
 {
 	gboolean ret;
 	GtkWidget *previous_focus_child;
-	PanelApplet *applet;
-
-	g_return_val_if_fail (PANEL_IS_APPLET (widget), FALSE);
-
-	applet = PANEL_APPLET (widget);
-	if (applet->priv->moving_focus_out) {
-		/*
-		 * Applet will retain focus if there is nothing else on the
-		 * panel to get focus
-		 */
-		applet->priv->moving_focus_out = FALSE;
-		return FALSE;
-	}
 
 	previous_focus_child = gtk_container_get_focus_child (GTK_CONTAINER (widget));
 	if (!previous_focus_child && !gtk_widget_has_focus (widget)) {
@@ -1259,20 +1036,6 @@ panel_applet_focus (GtkWidget        *widget,
 	return ret;
 }
 
-static void
-panel_applet_move_focus_out_of_applet (PanelApplet      *applet,
-				       GtkDirectionType  dir)
-{
-	GtkWidget *toplevel;
-
-	applet->priv->moving_focus_out = TRUE;
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (applet));
-	g_return_if_fail (toplevel);
-
-	gtk_widget_child_focus (toplevel, dir);
-	applet->priv->moving_focus_out = FALSE;
-}
-
 static void
 panel_applet_get_property (GObject    *object,
 			   guint       prop_id,
@@ -1340,8 +1103,7 @@ panel_applet_set_property (GObject      *object,
 		 * panel_applet_constructed() with a proper message if this is
 		 * the case. */
 		if (applet->priv->closure)
-			g_closure_set_marshal (applet->priv->closure,
-					       panel_applet_marshal_BOOLEAN__STRING);
+			g_closure_set_marshal (applet->priv->closure, g_cclosure_marshal_generic);
 		break;
 	case PROP_CONNECTION:
 		applet->priv->connection = g_value_dup_object (value);
@@ -1372,19 +1134,6 @@ panel_applet_set_property (GObject      *object,
 	}
 }
 
-static void
-add_tab_bindings (GtkBindingSet   *binding_set,
-		  GdkModifierType  modifiers,
-		  GtkDirectionType direction)
-{
-	gtk_binding_entry_add_signal (binding_set, GDK_KEY_Tab, modifiers,
-				      "move_focus_out_of_applet", 1,
-				      GTK_TYPE_DIRECTION_TYPE, direction);
-	gtk_binding_entry_add_signal (binding_set, GDK_KEY_KP_Tab, modifiers,
-				      "move_focus_out_of_applet", 1,
-				      GTK_TYPE_DIRECTION_TYPE, direction);
-}
-
 static void
 panel_applet_setup (PanelApplet *applet)
 {
@@ -1483,15 +1232,12 @@ panel_applet_class_init (PanelAppletClass *klass)
 {
 	GObjectClass   *gobject_class = (GObjectClass *) klass;
 	GtkWidgetClass *widget_class = (GtkWidgetClass *) klass;
-	GtkBindingSet *binding_set;
 
 	gobject_class->get_property = panel_applet_get_property;
 	gobject_class->set_property = panel_applet_set_property;
 	gobject_class->constructed = panel_applet_constructed;
         gobject_class->finalize = panel_applet_finalize;
 
-	klass->move_focus_out_of_applet = panel_applet_move_focus_out_of_applet;
-
 	widget_class->button_press_event = panel_applet_button_press;
 	widget_class->composited_changed = panel_applet_composited_changed;
 	widget_class->key_press_event = panel_applet_key_press_event;
@@ -1627,35 +1373,33 @@ panel_applet_class_init (PanelAppletClass *klass)
 			      1,
 			      G_TYPE_UINT);
 
-        /**
-         * PanelApplet::move-focus-out-of-applet: (skip)
-         * @applet: the #PanelApplet which emitted the signal.
-         * @direction: the move direction.
-         *
-	 * Emitted when the focus is moved out of @applet. This is an
-	 * implementation detail.
-         **/
-	panel_applet_signals [MOVE_FOCUS_OUT_OF_APPLET] =
-                g_signal_new ("move_focus_out_of_applet",
-                              G_TYPE_FROM_CLASS (klass),
-                              G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
-                              G_STRUCT_OFFSET (PanelAppletClass, move_focus_out_of_applet),
-                              NULL,
-			      NULL,
-                              g_cclosure_marshal_VOID__ENUM,
-                              G_TYPE_NONE,
-			      1,
-			      GTK_TYPE_DIRECTION_TYPE);
-
-	binding_set = gtk_binding_set_by_class (gobject_class);
-	add_tab_bindings (binding_set, 0, GTK_DIR_TAB_FORWARD);
-	add_tab_bindings (binding_set, GDK_SHIFT_MASK, GTK_DIR_TAB_BACKWARD);
-	add_tab_bindings (binding_set, GDK_CONTROL_MASK, GTK_DIR_TAB_FORWARD);
-	add_tab_bindings (binding_set, GDK_CONTROL_MASK | GDK_SHIFT_MASK, GTK_DIR_TAB_BACKWARD);
-
 	gtk_widget_class_set_css_name (widget_class, "panel-applet");
 }
 
+static GdkEvent *
+button_press_event_new (PanelApplet *applet,
+                        guint        button,
+                        guint        time)
+{
+  GdkDisplay *display;
+  GdkSeat *seat;
+  GdkDevice *device;
+  GdkEvent *event;
+
+  display = gdk_display_get_default ();
+  seat = gdk_display_get_default_seat (display);
+  device = gdk_seat_get_pointer (seat);
+
+  event = gdk_event_new (GDK_BUTTON_PRESS);
+
+  event->button.time = time;
+  event->button.button = button;
+
+  gdk_event_set_device (event, device);
+
+  return event;
+}
+
 static void
 method_call_cb (GDBusConnection       *connection,
 		const gchar           *sender,
@@ -1667,13 +1411,17 @@ method_call_cb (GDBusConnection       *connection,
 		gpointer               user_data)
 {
 	PanelApplet *applet = PANEL_APPLET (user_data);
+	GdkEvent *event;
 
 	if (g_strcmp0 (method_name, "PopupMenu") == 0) {
 		guint button;
 		guint time;
 
 		g_variant_get (parameters, "(uu)", &button, &time);
-		panel_applet_menu_popup (applet, button, time);
+
+		event = button_press_event_new (applet, button, time);
+		panel_applet_menu_popup (applet, event);
+		gdk_event_free (event);
 
 		g_dbus_method_invocation_return_value (invocation, NULL);
 	} else if (g_strcmp0 (method_name, "PopupEditMenu") == 0) {
@@ -1681,7 +1429,10 @@ method_call_cb (GDBusConnection       *connection,
 		guint time;
 
 		g_variant_get (parameters, "(uu)", &button, &time);
-		panel_applet_edit_menu_popup (applet, button, time);
+
+		event = button_press_event_new (applet, button, time);
+		panel_applet_edit_menu_popup (applet, event);
+		gdk_event_free (event);
 
 		g_dbus_method_invocation_return_value (invocation, NULL);
 	}
